@@ -37,19 +37,24 @@
 /* Top level file for synthesizer                                        */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdio.h>
+#include <cstdio>
 #include "EST_unix.h"
 #include "EST_Pathname.h"
-#include <stdlib.h>
+#include <cstdlib>
 #include "festival.h"
 #include "festivalP.h"
 #include "siod.h"
 #include "ModuleDescription.h"
 
-static void festival_lisp_funcs(void);
-static void festival_lisp_vars(void);
-static void festival_banner(void);
-static void festival_load_default_files(void);
+void festival_lisp_funcs(void);
+void festival_lisp_vars(void);
+void festival_banner(void);
+void festival_load_default_files(void);
+
+// Visual studio 2012 needs this
+#ifdef SYSTEM_IS_WIN32
+#define snprintf _snprintf
+#endif
 
 #define _S_S_S(S) #S
 #define STRINGIZE(S) _S_S_S(S)
@@ -84,22 +89,32 @@ LISP describe_module(LISP lname, LISP lstream);
 LISP describe_all_modules(void);
 #endif
 
+static int festival_initialized = 0;
+
 void festival_initialize(int load_init_files,int heap_size)
 {
     // all initialisation
 
-    siod_init(heap_size);
-    siod_est_init();		// add support for EST objects
-    siod_fringe_init();		// and for talking to fringe.
-
-    siod_prog_name = "festival";
-    cdebug = new ofstream("/dev/null");  // This wont work on Win/NT
-    stddebug = fopen("/dev/null","w");
-
-    festival_lisp_vars();
-    festival_lisp_funcs();
-    if (load_init_files)
-	festival_load_default_files();
+    if (! festival_initialized )
+    {
+	siod_init(heap_size);
+	siod_est_init();		// add support for EST objects
+	/* siod_fringe_init();	*/	// and for talking to fringe.
+	
+	siod_prog_name = "festival";
+	cdebug = new ofstream("/dev/null");  // This wont work on Win/NT
+	stddebug = fopen("/dev/null","w");
+	
+	festival_lisp_vars();
+	festival_lisp_funcs();
+	if (load_init_files)
+	    festival_load_default_files();
+	festival_initialized = TRUE;
+    }
+    else
+    {
+	cerr << "festival_initialize() called more than once" << endl;
+    }
 
     return;
 }
@@ -145,7 +160,7 @@ int festival_load_file(const EST_String &fname)
     EST_String b;
     b = EST_String("(load ")+quote_string(fname,"\"","\\",1)+")";
     // I used to do the above without the b intermediate variable
-    // but that caused a crash for some compilers on some machines 
+    // but that caused a crash for some compilers on some machines
     return festival_eval_command(b);
 }
 
@@ -167,9 +182,10 @@ int festival_eval_command(const EST_String &command)
     }
     else
     {
-	EST_String ll = command;  // copy it;
-	l = read_from_string((char *)ll);
-	leval(l,NIL); 
+        char *w = wstrdup((const char *)command);  // copy it;
+	l = read_from_string(w);
+	leval(l,NIL);
+        wfree(w);
 	rvalue = TRUE;
     }
 
@@ -191,21 +207,24 @@ void festival_tidy_up(void)
     
 }
 
-static void festival_banner(void)
+void festival_banner(void)
 {
     /* Print out a banner, copyright and version */
     
     if (siod_get_lval("hush_startup",NULL) == NIL)
     {
 	EST_Litem *t;
-	cout << STRINGIZE(FTNAME) << " " << 
+	cout << "\n" << STRINGIZE(FTNAME) << " " << 
 	    festival_version << endl;
-	cout << "Copyright (C) University of Edinburgh, 1996-2001. " <<
+	cout << "Copyright (C) University of Edinburgh, 1996-2010. " <<
 	    "All rights reserved." << endl;
-	cout << "For details type `(festival_warranty)'" << endl;
 	if (sub_copyrights.length() > 0)
-	    for (t = sub_copyrights.head(); t != 0; t = next(t))
+        {
+            cout << "\n";
+	    for (t = sub_copyrights.head(); t != 0; t = t->next())
 		cout << sub_copyrights.item(t);
+        }
+	cout << "For details type `(festival_warranty)'" << endl;
     }
 }
 
@@ -282,7 +301,7 @@ static LISP lisp_debug_output(LISP arg)
     return NIL;
 }
 
-static void festival_load_default_files(void)
+void festival_load_default_files(void)
 {
     // Load in default files, init.scm. Users ~/.festivalrc 
     // (or whatever you wish to call it) is loaded by init.scm
@@ -298,7 +317,7 @@ static void festival_load_default_files(void)
 
 }
 
-static void festival_lisp_vars(void)
+void festival_lisp_vars(void)
 {
     // set up specific lisp variables 
     EST_TokenStream ts;
@@ -331,6 +350,8 @@ static void festival_lisp_vars(void)
 	proclaim_module("freebsd16audio");
     if (linux16_supported)
 	proclaim_module("linux16audio");
+    if (macosx_supported)
+	proclaim_module("macosxaudio");
     if (win32audio_supported)
 	proclaim_module("win32audio");
     if (mplayer_supported)
@@ -346,7 +367,9 @@ static void festival_lisp_vars(void)
     //  Modify my PATH to include these directories
     siod_set_lval("etc-path",cons(rintern(etcdir),
 				  cons(rintern(etcdircommon),NIL)));
-    char *path = getenv("PATH");
+    const char *path = getenv("PATH");
+    if (path == 0)
+	path = "";
     char *newpath = walloc(char,1024+strlen(path)+strlen(etcdir)+
 			   strlen(etcdircommon));
     sprintf(newpath,"PATH=%s:%s:%s",path,etcdir,etcdircommon);
@@ -369,7 +392,160 @@ LISP l_lr_predict(LISP si, LISP lr_model);
 void festival_unitdb_init(void);
 LISP Gen_Viterbi(LISP utt);
 
-static void festival_lisp_funcs(void)
+inline int utf8_sequence_length(char c0) {
+  // Get the expected length of UTF8 sequence given its most
+  // significant byte
+  return (( 0xE5000000 >> (( c0 >> 3 ) & 0x1E )) & 3 ) + 1;
+}
+
+LISP utf8_explode(LISP name) {
+    /* return a list of utf-8 characters as strings */
+    LISP chars = NIL;
+    const char *str = (const char*)get_c_string(name);
+    char utf8char[5];
+    char c0;
+    int charlength;
+
+    while ((c0 = *str)) {
+      charlength = utf8_sequence_length(c0);
+      snprintf(utf8char, charlength + 1, "%s", str);
+      chars = cons(strcons(charlength, utf8char), chars);
+      str += charlength;
+    }
+
+    return reverse(chars);
+}
+
+
+int utf8_ord(const char *utf8_seq) {
+  unsigned int len;
+  int ord;
+
+  unsigned char c0, c1, c2, c3;  // Potential bytes in the UTF8 symbol
+  c0 = utf8_seq[0];
+  len = utf8_sequence_length(c0);
+
+  // Make sure the string sequence we received matches with the
+  // expected length, and that the expected length is nonzero.
+  if ( (len == 0) ||
+       (len != strlen(utf8_seq))) {
+    return -1;
+  }
+
+  if (len == 1) {
+    // ASCII sequence.
+    return c0;
+  }
+
+  c1 = utf8_seq[1];
+  if (len == 2) {
+    ord = ((c0 & 0x1F) << 6) | (c1 & 0x3F);
+    if (ord < 0x80)
+      return -1;
+    return ord;
+  }
+
+  c2 = utf8_seq[2];
+  if (len == 3) {
+    if ((c2 & 0xC0) != 0x80)
+      return -1;
+    ord = ((c0 & 0x0F) << 12) | ((c1 & 0x3F) << 6) | (c2 & 0x3F);
+    if (ord < 0x800 ||
+        (ord >= 0xD800 && ord <= 0xDFFF))
+      return -1;
+    return ord;
+  }
+
+  c3 = utf8_seq[3];
+  if (len == 4) {
+    if ((c3 & 0xC0) != 0x80)
+      return -1;
+    ord =
+        ((c0 & 0x7) << 18) | ((c1 & 0x3F) << 12) |
+        ((c2 & 0x3F) << 6) | (c3 & 0x3F);
+    if (ord < 0x10000 || ord > 0x10FFFF)
+      return -1;
+    return ord;
+  }
+
+  return -1;
+}
+
+LISP utf8_ord_wrapper(LISP utf8_char) {
+  const char *ch = (const char *)get_c_string(utf8_char);
+  return lisp_val(utf8_ord(ch));
+}
+
+int utf8_chr(int ord, char* utf8char) {
+  unsigned int utf8len;
+  int i = 0;
+
+  if (ord < 0x80) {
+    utf8len = 1;
+  } else if (ord < 0x800) {
+    utf8len = 2;
+  } else if (ord <= 0xFFFF) {
+    utf8len = 3;
+  } else if (ord <= 0x200000) {
+    utf8len = 4;
+  } else {
+    // Replace invalid character with FFFD
+    utf8len = 2;
+    ord = 0xFFFD;
+  }
+
+  i = utf8len;  // Index into utf8char
+  utf8char[i--] = 0;
+
+  switch (utf8len) {
+    // These fallthrough deliberately
+    case 6:
+      utf8char[i--] = (ord | 0x80) & 0xBF;
+      ord >>= 6;
+    case 5:
+      utf8char[i--] = (ord | 0x80) & 0xBF;
+      ord >>= 6;
+    case 4:
+      utf8char[i--] = (ord | 0x80) & 0xBF;
+      ord >>= 6;
+    case 3:
+      utf8char[i--] = (ord | 0x80) & 0xBF;
+      ord >>= 6;
+    case 2:
+      utf8char[i--] = (ord | 0x80) & 0xBF;
+      ord >>= 6;
+    case 1:
+      switch (utf8len) {
+        case 0:
+        case 1:
+          utf8char[i--] = ord;
+          break;
+        case 2:
+          utf8char[i--] = ord | 0xC0;
+          break;
+        case 3:
+          utf8char[i--] = ord | 0xE0;
+          break;
+        case 4:
+          utf8char[i--] = ord | 0xF0;
+      }
+  }
+  return utf8len;
+}
+
+LISP utf8_chr_wrapper(LISP ord) {
+  char ch[5];
+  int utf8len;
+
+  utf8len = utf8_chr(get_c_int(ord), ch);
+  if (utf8len == 0) {
+    return NIL;
+  }
+
+  return strcons(utf8len, ch);
+}
+
+void festival_lisp_funcs(void)
 {
     // declare festival specific Lisp functions 
     
@@ -399,7 +575,17 @@ static void festival_lisp_funcs(void)
  "(debug_output ARG)\n\
   If ARG is non-nil cause all future debug output to be sent to cerr,\n\
   otherwise discard it (send it to /dev/null).");
+    init_subr_1("utf8explode", utf8_explode,
+ "(utf8explode utf8string)\n\
+  Returns a list of utf-8 characters in given string.");
     
+    init_subr_1("utf8ord", utf8_ord_wrapper,
+ "(utf8ord utf8char)\n\
+  Returns the decimal value utf8 character. -1 if character is invalid.");
+    init_subr_1("utf8chr", utf8_chr_wrapper,
+ "(utf8chr ord)\n\
+  Returns the utf8 character corresponding to given decimal value. Invalid characters are replaced by 0xFFFD.");
+
     init_subr_2("wagon",l_wagon,
  "(wagon ITEM TREE)\n\
   Apply the CART tree TREE to ITEM.  This returns the full\n\
@@ -409,7 +595,7 @@ static void festival_lisp_funcs(void)
  "(lr_predict ITEM LRMODEL)\n\
   Apply the linear regression model LRMODEL to ITEM in.  This\n\
   returns float value by summing the product of the coeffients and values\n\
-  returned by the specificed features in ITEM. [see Linear regression]");
+  returned by the specified features in ITEM. [see Linear regression]");
     init_subr_1("Gen_Viterbi",Gen_Viterbi,
  "(Gen_Viterbi UTT)\n\
   Applies viterbi search algorithm based on the parameters in\n\
