@@ -34,8 +34,11 @@
 #ifndef __MODULEDESCRIPTION_H__
 #define __MODULEDESCRIPTION_H__
 
-#include <stdio.h>
-#include <iostream.h>
+#include <cstdio>
+#include <iostream>
+
+using namespace std;
+
 #include "EST_String.h"
 
 /** Machine readable descriptions of modules. Useful for help messages
@@ -44,7 +47,7 @@
   * This is a struct rather than a class so that it can be initialised
   * in the source of the module. 
   * @author Richard Caley <rjc@cstr,ed,ac,uk>
-  * @version $Id: ModuleDescription.h,v 1.9 1999/05/30 13:38:52 awb release $
+  * @version $Id: ModuleDescription.h,v 1.4 2014/11/25 14:32:04 robert Exp $
   */
 
 struct ModuleDescription {
@@ -139,7 +142,9 @@ struct ModuleDescription {
 /// Output operator for descriptions.
 ostream &operator << (ostream &stream, const ModuleDescription &desc);
 
-VAL_REGISTER_CLASS_DCLS(moddesc,ModuleDescription)
-SIOD_REGISTER_CLASS_DCLS(moddesc,ModuleDescription)
+//VAL_REGISTER_CLASS_DCLS(moddesc,ModuleDescription) // clang/llvm complains about this (Rob)
+//SIOD_REGISTER_CLASS_DCLS(moddesc,ModuleDescription)
+VAL_REGISTER_TYPE_DCLS(moddesc,ModuleDescription)
+SIOD_REGISTER_TYPE_DCLS(moddesc,ModuleDescription)
 
 #endif
