@@ -37,7 +37,7 @@
 /* Duration averages and default and tree                                */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdio.h>
+#include <cstdio>
 #include "festival.h"
 #include "durationP.h"
 
@@ -54,7 +54,7 @@ LISP FT_Duration_Ave_Utt(LISP utt)
 
     ph_durs = siod_get_lval("phoneme_durations","no phoneme durations");
 
-    for (s=u->relation("Segment")->first(); s != 0; s = next(s))
+    for (s=u->relation("Segment")->first(); s != 0; s = inext(s))
     {
 	ldur = siod_assoc_str(s->name(),ph_durs);
 	stretch = dur_get_stretch_at_seg(s);
@@ -83,7 +83,7 @@ LISP FT_Duration_Def_Utt(LISP utt)
 
     *cdebug << "Duration Default module\n";
 
-    for (s=u->relation("Segment")->first(); s != 0; s = next(s))
+    for (s=u->relation("Segment")->first(); s != 0; s = inext(s))
     {
 	stretch = dur_get_stretch_at_seg(s);
 	end += 0.100*stretch;
@@ -106,7 +106,7 @@ LISP FT_Duration_Tree_Utt(LISP utt)
 
     tree = siod_get_lval("duration_cart_tree","no duration cart tree");
 
-    for (s=u->relation("Segment")->first(); s != 0; s = next(s))
+    for (s=u->relation("Segment")->first(); s != 0; s = inext(s))
     {
 	pdur = wagon_predict(s,tree);
 	stretch = dur_get_stretch_at_seg(s);
@@ -137,13 +137,14 @@ LISP FT_Duration_Tree_ZScores_Utt(LISP utt)
     float end=0.0, dur,stretch;
     LISP tree,dur_info,ph_info;
     float pdur;
+    float ave, std;
 
     *cdebug << "Duration Tree ZScores module\n";
 
     tree = siod_get_lval("duration_cart_tree","no duration cart tree");
     dur_info = siod_get_lval("duration_ph_info","no duration phone info");
 
-    for (s=u->relation("Segment")->first(); s != 0; s = next(s))
+    for (s=u->relation("Segment")->first(); s != 0; s = inext(s))
     {
 	pdur = wagon_predict(s,tree);
 	ph_info = siod_assoc_str(s->name(),dur_info);
@@ -151,16 +152,22 @@ LISP FT_Duration_Tree_ZScores_Utt(LISP utt)
 	if (ph_info == NIL)
 	{
 	    cerr << "Phoneme: " << s->name() << " has no duration info\n";
-	    festival_error();
+            ave = 0.080;
+            std = 0.020;
 	}
+        else
+        {
+            ave = PH_AVE(ph_info);
+            std = PH_STD(ph_info);
+        }
 	if ((pdur > 3) || (pdur < -3))
 	{
-	    cwarn << "Duration tree extreme for " << s->name() << 
-		" " << pdur << endl;
+            //	    cerr << "Duration tree extreme for " << s->name() << 
+            //		" " << pdur << endl;
 	    pdur = ((pdur < 0) ? -3 : 3);
 	}
 	s->set("dur_factor",pdur);
-	dur = PH_AVE(ph_info) + (pdur*PH_STD(ph_info));
+	dur = ave + (pdur*std);
 	dur *= stretch;
 	if (dur < 0.010)
 	    dur = 0.010;  // just in case it goes wrong
