@@ -43,11 +43,11 @@
 /* described in Black and Hunt ICSLP96)                                  */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdio.h>
+#include <cstdio>
 #include "festival.h"
 #include "intonation.h"
 
-enum lr_tpos {tp_start, tp_mid, tp_end};
+enum lr_tpos {tp_start, tp_left, tp_mid, tp_right, tp_end};
 
 static EST_String accent_specified(EST_Item *s);
 static EST_String tone_specified(EST_Item *s);
@@ -60,6 +60,9 @@ static void add_target_at(EST_Utterance *u, EST_Item *seg,
 static float apply_lr_model(LISP model, EST_FVector &feats);
 static void find_feat_values(EST_Item *s, LISP model,EST_FVector &feats);
 
+static LISP Intonation_Endtone_Tree_Utt(LISP utt);  // ... mh 99-08-06
+static LISP Intonation_Accent_Tree_Utt(LISP utt);
+
 static float target_f0_mean = 0.0;
 static float target_f0_std = 1.0;
 static float model_f0_mean = 0.0;
@@ -74,24 +77,52 @@ LISP FT_Intonation_Tree_Utt(LISP utt)
     // For each syllable predict intonation events.  Potentially
     // two forms, accents and ent tones
     EST_Utterance *u = get_c_utt(utt);
-    EST_Item *s;
-    EST_String paccent,ptone;
-    LISP accent_tree, endtone_tree;
-
-    accent_tree = siod_get_lval("int_accent_cart_tree","no accent tree");
-    endtone_tree = siod_get_lval("int_tone_cart_tree","no tone cart tree");
 
     u->create_relation("IntEvent");
     u->create_relation("Intonation");
 
-    for (s=u->relation("Syllable")->first(); s != 0; s=next(s))
+    utt = Intonation_Endtone_Tree_Utt(utt);
+    utt = Intonation_Accent_Tree_Utt(utt);
+
+    return utt;
+}
+
+LISP Intonation_Accent_Tree_Utt(LISP utt)
+{
+    // For each syllable predict intonation events.
+    // here only accents
+    EST_Utterance *u = get_c_utt(utt);
+    EST_Item *s;
+    EST_String paccent;
+    LISP accent_tree;
+
+    accent_tree = siod_get_lval("int_accent_cart_tree","no accent tree");
+
+    for (s=u->relation("Syllable")->first(); s != 0; s=inext(s))
     {
 	if ((paccent = accent_specified(s)) == "0") // check if pre-specified
-	    paccent = wagon_predict(s,accent_tree);
+	    paccent = (EST_String)wagon_predict(s,accent_tree);
 	if (paccent != "NONE")
 	    add_IntEvent(u,s,paccent);
+    }
+    return utt;
+}
+
+LISP Intonation_Endtone_Tree_Utt(LISP utt)
+{
+    // For each syllable predict intonation events.
+    // here only endtones
+    EST_Utterance *u = get_c_utt(utt);
+    EST_Item *s;
+    EST_String ptone;
+    LISP endtone_tree;
+
+    endtone_tree = siod_get_lval("int_tone_cart_tree","no tone cart tree");
+
+    for (s=u->relation("Syllable")->first(); s != 0; s=inext(s))
+    {
 	if ((ptone = tone_specified(s)) == "0")
-	    ptone = wagon_predict(s,endtone_tree);
+	    ptone = (EST_String)wagon_predict(s,endtone_tree);
 	if (ptone != "NONE")
 	    add_IntEvent(u,s,ptone);
     }
@@ -107,18 +138,18 @@ static EST_String accent_specified(EST_Item *s)
     EST_Item *token = parent(word,"Token");
     EST_String paccent("0");
     if (token)
-	paccent = ffeature(token,"accent");
+	paccent = (EST_String)ffeature(token,"accent");
 	
     if (paccent == "0")
     {
-	paccent = ffeature(word,"accent");
+	paccent = (EST_String)ffeature(word,"accent");
 	if (paccent == "0")
 	    return paccent;
     }
     if (ffeature(s,"stress") == "1")
     {   // only goes on first stressed syllable
 	EST_Item *p;
-	for (p=prev(s,"SylStructure"); p != 0; p=prev(p))
+	for (p=iprev(as(s,"SylStructure")); p != 0; p=iprev(p))
 	    if (ffeature(s,"stress") == "1")
 		return "NONE";  // specified but not on this syllable
 	return paccent;  // first stressed syl in word
@@ -139,15 +170,15 @@ static EST_String tone_specified(EST_Item *s)
     EST_Item *token = parent(word,"Token");
     EST_String ptone("0");
     if (token)
-	ptone = ffeature(token,"tone");
+	ptone = (EST_String)ffeature(token,"tone");
 
     if (ptone == "0")
     {
-	ptone = ffeature(word,"tone");
+	ptone = (EST_String)ffeature(word,"tone");
 	if (ptone == "0")
 	    return ptone;
     }
-    if (next(ss) == 0)  // final syllable in word
+    if (inext(ss) == 0)  // final syllable in word
 	return ptone;
     else
 	return "NONE";  // pre-specified but inappropriate syllable in word
@@ -172,7 +203,7 @@ LISP FT_Int_Targets_LR_Utt(LISP utt)
     EST_FVector feats;
     feats.resize(siod_llength(start_lr));
 
-    for (s=u->relation("Syllable")->first(); s != 0; s=next(s))
+    for (s=u->relation("Syllable")->first(); s != 0; s=inext(s))
     {
 	find_feat_values(s,start_lr,feats);
 	pstart = apply_lr_model(start_lr,feats);
@@ -197,6 +228,60 @@ LISP FT_Int_Targets_LR_Utt(LISP utt)
 
 }
 
+LISP FT_Int_Targets_LR_5_Utt(LISP utt)
+{
+  // Predict F0 targets using Linear regression
+  // This version uses 5 points rather than 3.
+   EST_Utterance *u = get_c_utt(utt);
+    EST_Item *s;
+    float pstart, pleft, pmid, pright, pend;
+    LISP start_lr, left_lr, mid_lr, right_lr, end_lr;
+
+    init_int_lr_params();
+    // Note the models must *all* be the same size
+    start_lr = siod_get_lval("f0_lr_start","no f0 start lr model");
+    left_lr = siod_get_lval("f0_lr_left","no f0 left lr model");
+    mid_lr = siod_get_lval("f0_lr_mid","no f0 mid lr model");
+    right_lr = siod_get_lval("f0_lr_right","no f0 right lr model");
+    end_lr = siod_get_lval("f0_lr_end","no f0 end lr model");
+    
+    u->create_relation("Target");
+    pend = 0;
+    EST_FVector feats;
+    feats.resize(siod_llength(start_lr));
+
+    for (s=u->relation("Syllable")->first(); s != 0; s=inext(s))
+    {
+	find_feat_values(s,start_lr,feats);
+	pstart = apply_lr_model(start_lr,feats);
+	pstart = MAP_F0(pstart);
+	if (after_pause(s))
+	    add_target_at(u,daughter1(s,"SylStructure"),pstart,tp_start);
+	else
+	    add_target_at(u,daughter1(s,"SylStructure"),
+			  (pstart+pend)/2.0,tp_start);
+
+	pleft = apply_lr_model(left_lr,feats);
+	pleft = MAP_F0(pleft);
+	add_target_at(u,vowel_seg(s),pleft,tp_left);
+	pmid = apply_lr_model(mid_lr,feats);
+	pmid = MAP_F0(pmid);
+	add_target_at(u,vowel_seg(s),pmid,tp_mid);
+	pright = apply_lr_model(right_lr,feats);
+	pright = MAP_F0(pright);
+	add_target_at(u,vowel_seg(s),pright,tp_right);
+
+	pend = apply_lr_model(end_lr,feats);
+	pend = MAP_F0(pend);
+	if (before_pause(s))
+	    add_target_at(u,daughtern(s,"SylStructure"),pend,tp_end);
+    }
+
+    return utt;
+
+}
+
+
 #define FFEATURE_NAME(X) (get_c_string(car(X)))
 #define FFEATURE_WEIGHT(X) (get_c_float(car(cdr(X))))
 #define FFEATURE_MAPCLASS(X) (car(cdr(cdr(X))))
@@ -263,14 +348,24 @@ static void add_target_at(EST_Utterance *u, EST_Item *seg,
 	    << endl;
 	return;
     }
-
-    if (pos == tp_start)
-	add_target(u,seg,ffeature(seg,"segment_start").Float(),val);
-    else if (pos == tp_mid)
-	add_target(u,seg,ffeature(seg,"segment_mid").Float(),val);
-    else if (pos == tp_end)
-	add_target(u,seg,seg->F("end"),val);
-    else
+  
+  if (pos == tp_start)
+    add_target(u,seg,ffeature(seg,"segment_start").Float(),val);
+  else if (pos == tp_left)
+    add_target(u,seg,
+	       0.5*(ffeature(seg,"segment_mid").Float()+
+		    ffeature(seg,"segment_start").Float()),
+	       val);
+  else if (pos == tp_mid)
+    add_target(u,seg,ffeature(seg,"segment_mid").Float(),val);
+  else if (pos == tp_right)
+    add_target(u,seg,
+	       0.5*(ffeature(seg,"segment_mid").Float()+
+		    seg->F("end")),
+	       val);
+  else if (pos == tp_end)
+    add_target(u,seg,seg->F("end"),val);
+  else
     {
 	cerr << "add_target_at: unknown position type\n";
 	festival_error();
@@ -281,13 +376,13 @@ static int after_pause(EST_Item *s)
 {
     // TRUE if segment immediately previous to this is a silence
     EST_Item *p;
-    if (prev(s) == 0)
+    if (iprev(s) == 0)
 	return TRUE;
     EST_Item *ss = s->as_relation("SylStructure");
-    if (prev(s) == prev(ss))
+    if (iprev(s) == iprev(ss))
 	return FALSE;
 
-    p = daughter1(ss)->as_relation("Segment")->prev();
+    p = iprev(daughter1(ss)->as_relation("Segment"));
     if (p == 0)
 	return TRUE;
     else if (ph_is_silence(p->name()))
@@ -299,10 +394,10 @@ static int after_pause(EST_Item *s)
 static int before_pause(EST_Item *s)
 {
     // TRUE is segment immediately after this is a silence
-    if (next(s) == 0)
+    if (inext(s) == 0)
 	return TRUE;
     EST_Item *ss = s->as_relation("SylStructure");
-    EST_Item *n = daughtern(ss)->as_relation("Segment")->next();
+    EST_Item *n = inext(daughtern(ss)->as_relation("Segment"));
     if (ph_is_silence(n->name()))
 	return TRUE;
     else 
@@ -314,7 +409,7 @@ static EST_Item *vowel_seg(EST_Item *syl)
     // return related to vowel segment
     EST_Item *p;
 
-    for (p=daughter1(syl,"SylStructure"); p != 0; p=next(p))
+    for (p=daughter1(syl,"SylStructure"); p != 0; p=inext(p))
 	if (ph_is_vowel(p->name()))
 	    return p;
 
