@@ -45,7 +45,7 @@
 /* when you don't have the language specific version.                    */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdio.h>
+#include <cstdio>
 #include "festival.h"
 #include "text.h"
 
@@ -169,7 +169,7 @@ void tts_file_raw(LISP filename)
     lutt = tts_chunk_stream(ts,tts_raw_token,tts_raw_utt,eou_tree,0);
     
     // The last one is returned because the chunker doesn't know if this
-    // is truely the end of an utterance or not, but here we do know.
+    // is truly the end of an utterance or not, but here we do know.
     tts_raw_utt(lutt);
 
     ts.close();
@@ -198,9 +198,12 @@ static void tts_raw_utt(LISP utt)
 		cons(rintern("tts_hooks"),
 		     cons(lutt,NIL)));
 
+
+
     lutt = cons(rintern("set!"),
 		cons(rintern("utt_tts"),
 		     cons(lutt,NIL)));
+
     // Synth and Play it
     lutt = leval(lutt,NIL);
     user_gc(NIL);
@@ -208,7 +211,7 @@ static void tts_raw_utt(LISP utt)
 
 LISP new_token_utt(void)
 {
-    // An emtpy utterance ready to take Tokens
+    // An empty utterance ready to take Tokens
     EST_Utterance *u = new EST_Utterance;
     u->f.set("type","Tokens");
     u->create_relation("Token");
@@ -238,7 +241,7 @@ LISP tts_chunk_stream(EST_TokenStream &ts,
 	t = ts.get();
 	tok = add_token(u,t);
 	app_tok(tok);     // do what you do with the token
-	ebo = prev(tok,"Token");  // end but one token
+	ebo = iprev(as(tok,"Token"));  // end but one token
 	if ((ebo != 0) &&
 	    (wagon_predict(ebo,eou_tree) == 1))
 	{
@@ -254,6 +257,22 @@ LISP tts_chunk_stream(EST_TokenStream &ts,
     return utt;
 }
 
+#if 0
+LISP memon(void)
+{
+    printf("memon\n");
+    putenv("MALLOC_TRACE=mallfile");
+    mtrace();
+    return NIL;
+}
+
+LISP memoff(void)
+{
+    muntrace();
+    printf("memoff\n");
+    return NIL;
+}
+#endif
 
 void festival_Text_init(void)
 {
@@ -266,9 +285,15 @@ void festival_Text_init(void)
   Low level access to tts function, you probably want to use the function\n\
   tts rather than this one.  Render data in FILE as speech.  Respect\n\
   MODE.  Currently modes are defined through the variable tts_text_modes.");
+#if 0
+    init_subr_0("memon",memon,
+		"(tts_file FILE MODE)");
+    init_subr_0("memoff",memoff,
+		"(tts_file FILE MODE)");
+#endif
     init_subr_3("extract_tokens",extract_tokens,
     "(extract_tokens FILE TOKENS OUTFILE)\n\
-  Find all occurences of TOKENS in FILE and output specified context around\n\
+  Find all occurrences of TOKENS in FILE and output specified context around\n\
   the token.  Results are appended to OUTFILE, if OUTFILE is nil, output\n\
   goes to stdout.");
 }
