@@ -46,7 +46,7 @@
 /*    exit function        run after mode is applied                     */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdio.h>
+#include <cstdio>
 #include "EST_unix.h"
 #include "festival.h"
 #include "text.h"
@@ -60,7 +60,7 @@ void tts_file_user_mode(LISP filename, LISP params)
 {
 
     volatile EST_String tmpname = make_tmp_filename();
-    volatile EST_String inname = get_c_string(filename);
+    volatile EST_String inname = (EST_String)get_c_string(filename);
     volatile EST_String filter;
     volatile EST_TokenStream ts;
     volatile LISP func;
@@ -152,7 +152,7 @@ void um_apply_filter(const EST_String &filtername,
     }
     else
     {
-	command = filtername + " " + infile + " > " + outfile;
+	command = filtername + " '" + infile + "' > " + outfile;
 	system(command);  // should test if this is successful or not
     }
 }
