@@ -5,15 +5,19 @@
 /*                       Copyright (c) 1996,1997                         */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -40,7 +44,10 @@
 /* Note only English tokenization has any substance at present           */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdio.h>
+#include <cstdio>
+
+using namespace std;
+
 #include "festival.h"
 #include "lexicon.h"
 #include "modules.h"
@@ -58,8 +65,8 @@ static EST_String remove_punct(const EST_String &tok);
 static int only_punc(const EST_String &tok);
 static LISP num_2_words(int iword);
 static LISP say_num_as_words(const EST_String &num);
-static LISP word_it(LISP utt,EST_Stream_Item &t,EST_String tok);
-static LISP builtin_word_it(LISP utt, EST_Stream_Item &token, EST_String tok);
+static LISP word_it(EST_Item *t,const EST_String tok);
+static LISP builtin_word_it(EST_Item *token, EST_String tok);
 static LISP say_as_letters(const EST_String &word);
 static LISP say_num_as_ordinal(const EST_String &num);
 static LISP say_num_as_year(const EST_String &num);
@@ -91,30 +98,29 @@ LISP FT_Any_Token_Utt(LISP utt)
 {
     // Language independent EST_Token to Word module.  Uses user specified
     // token to word function of simply creates a word for each token.
-    EST_Utterance *u = GETUTTVAL(utt);
-    LISP eou_tree,words,w;
-    EST_Stream_Item *t;
-    EST_Stream_Item *new_word;
+    EST_Utterance *u = get_c_utt(utt);
+    LISP words,w;
+    EST_Item *t;
+    EST_Item *new_word;
 
-    u->create_stream("Word");
-    eou_tree = siod_get_lval("eou_tree","No end of utterance tree");
     user_token_to_word_func = siod_get_lval("token_to_words",NULL);
+    u->create_relation("Word");
 
-    for (t=(u->stream("Token")).head(); t != 0; t = next(t))
+    for (t=u->relation("Token")->first(); t != 0; t = inext(t))
     {
 	if (user_token_to_word_func != NIL)
 	{
-	    words = word_it(utt,*t,t->name());
+	    words = word_it(t,t->name());
 	    for (w=words; w != NIL; w=cdr(w))
 	    {
-		new_word = add_word(*u,car(w));
-		link(*new_word,*t);
+		new_word = add_word(u,car(w));
+		append_daughter(t,"Token",new_word);
 	    }
 	}
 	else
 	{   // No user token_to_word function so just do it directly
-	    new_word = add_word(*u,t->name());
-	    link(*new_word,*t);
+	    new_word = add_word(u,t->name());
+	    append_daughter(t,"Token",new_word);
 	}
     }
     user_token_to_word_func = NIL;  // reset this
@@ -128,55 +134,55 @@ LISP FT_English_Token_Utt(LISP utt)
     // Tokens may go to zero or more words, tokens retain information 
     // about their punctuation and spacing on the page.
     //  Preceeding and succeeding punctuation become words
-    EST_Utterance *u = GETUTTVAL(utt);
-    EST_Stream_Item *t;
+    EST_Utterance *u = get_c_utt(utt);
+    EST_Item *t;
     LISP words,w,eou_tree,l;
-    EST_Stream_Item *new_word;
+    EST_Item *new_word;
 
     *cdebug << "Token module (English)" << endl;
 
-    u->create_stream("Word");
     eou_tree = siod_get_lval("eou_tree","No end of utterance tree");
     user_token_to_word_func = siod_get_lval("token_to_words",NULL);
+    u->create_relation("Word");
 
-    for (t=(u->stream("Token")).head(); t != 0; t = next(t))
+    for (t=u->relation("Token")->first(); t != 0; t = inext(t))
     {
-	words = word_it(utt,*t,t->name());
+	words = word_it(t,t->name());
 	// Initial punctuation becomes words
 	new_word = 0;
-	if ((t->feature("prepunctuation") != "0") &&
-	    (t->feature("prepunctuation") != ""))
+	if ((t->f("prepunctuation") != "0") &&
+	    (t->f("prepunctuation") != ""))
 	{
-	    l = symbolexplode(strintern(t->feature("prepunctuation").string()));
+	    l = symbolexplode(strintern(t->f("prepunctuation").string()));
 	    for (w=l; w != NIL; w=cdr(w))
 	    {
-		new_word = add_word(*u,car(w));
-		link(*new_word,*t);
+		new_word = add_word(u,car(w));
+		append_daughter(t,"Token",new_word);
 	    }
 	}
 
 	// Words become words
 	for (w=words; w != NIL; w=cdr(w))
 	{
-	    new_word = add_word(*u,car(w));
-	    link(*new_word,*t);
+	    new_word = add_word(u,car(w));
+	    append_daughter(t,"Token",new_word);
 	}
 
 	// final word gets punctuation marking
-	if ((new_word != 0) && (t->feature("punc") != "0"))
+	if ((new_word != 0) && (ffeature(t,"punc") != "0"))
 	{
-	    if ((t->feature("punc") == ".") &&
-		(wagon_predict(*u,*t,eou_tree) == 0))
+	    if ((ffeature(t,"punc") == ".") &&
+		(wagon_predict(t,eou_tree) == 0))
 	    {   // It wasn't a really punctuation mark
-		t->set_feature("punc","0");
+		t->set("punc","0");
 	    }
 	    else
 	    {
-		l = symbolexplode(strintern(t->feature("punc").string()));
+		l = symbolexplode(strintern(ffeature(t,"punc").string()));
 		for (w=l; w != NIL; w=cdr(w))
 		{
-		    new_word = add_word(*u,car(w));
-		    link(*new_word,*t);
+		    new_word = add_word(u,car(w));
+		    append_daughter(t,"Token",new_word);
 		}
 	    }
 	}
@@ -188,39 +194,7 @@ LISP FT_English_Token_Utt(LISP utt)
 
 }
 
-#if 0
-static EST_String punc2name(const EST_String &punc)
-{
-    // Change punctuation mark to more readable name 
-
-    if (punc == "")
-	return "none";
-    else if (punc == ",")
-	return "comma";
-    else if (punc == ".")
-	return "period";
-    else if (punc == "?")
-	return "qmark";
-    else if (punc == ":")
-	return "colon";
-    else if (punc == ";")
-	return "semicolon";
-    else if (punc == "\"")
-	return "quote";
-    else if (punc == "'")
-	return "squote";
-    else if (punc == "!")
-	return "exclaim";
-    else if (punc == "-")
-	return "hyphen";
-    else if (punc.length() > 1)
-	return "lots";
-    else
-	return "other";
-}
-#endif
-
-static LISP word_it(LISP utt, EST_Stream_Item &token, EST_String tok)
+static LISP word_it(EST_Item *token, const EST_String tok)
 {
     // The user may specify their own addition;a token to word rules
     // through the variable user_token_to_word_func if so we must
@@ -233,14 +207,13 @@ static LISP word_it(LISP utt, EST_Stream_Item &token, EST_String tok)
 
     if (user_token_to_word_func != NIL)            // check user's rules
 	return leval(cons(user_token_to_word_func,
-			  cons(utt,
-			       cons(siod_make_streamitem(&token),
-				    cons(tok_string,NIL)))),NIL);
+			  cons(siod(token),
+			       cons(tok_string,NIL))),NIL);
     else
-	return builtin_word_it(utt,token,tok);
+	return builtin_word_it(token,tok);
 }
 
-static LISP builtin_word_it(LISP utt, EST_Stream_Item &token, EST_String tok)
+static LISP builtin_word_it(EST_Item *token, EST_String tok)
 {
     // Return a list of words for this token 
     EST_String token_pos;
@@ -249,7 +222,7 @@ static LISP builtin_word_it(LISP utt, EST_Stream_Item &token, EST_String tok)
 	return NIL;
     else if (in_current_lexicon(downcase(tok),NIL))  // if in lexicon use as is
     {
-	if ((tok != token.name()) && // mainly to catch internal "a"
+	if ((tok != token->name()) && // mainly to catch internal "a"
 	    (tok.length() == 1))
 	{
 	    LISP let_pos = siod_get_lval("token.letter_pos",NULL);
@@ -260,7 +233,7 @@ static LISP builtin_word_it(LISP utt, EST_Stream_Item &token, EST_String tok)
 	else
 	    return cons(strintern(tok),NIL);
     }
-    else if ((token_pos = token.feature("token_pos")) == "ordinal")
+    else if ((token_pos = (EST_String)ffeature(token,"token_pos")) == "ordinal")
 	return say_num_as_ordinal(tok);
     else if (token_pos == "year")
 	return say_num_as_year(tok);
@@ -274,14 +247,14 @@ static LISP builtin_word_it(LISP utt, EST_Stream_Item &token, EST_String tok)
     else if (tok.matches(RXintcommaed))  // containing commas at thousands
     {
 	if (tok.contains("."))
-	    return word_it(utt,token,remove_punct(tok.before("."))+
-				    "."+tok.after("."));
+	    return word_it(token,remove_punct(tok.before("."))+
+			   "."+tok.after("."));
 	else
 	    return say_num_as_words(remove_punct(tok));
     }
     else if (tok.matches(RXapostropheS))
     {
-	return l_append(word_it(utt,token,tok.at(0,tok.length()-2)),
+	return append(word_it(token,tok.at(0,tok.length()-2)),
 			cons(strintern("'s"),NIL));
     }
     else if (tok.matches(numpointnum))
@@ -290,12 +263,14 @@ static LISP builtin_word_it(LISP utt, EST_Stream_Item &token, EST_String tok)
 	LISP ap = NIL;
 	int i;
 	for (i=0; i < afterpoint.length(); i++)
-	     ap = l_append(say_num_as_words(afterpoint.at(i,1)),ap);
-	return l_append(say_num_as_words(tok.before(".")),
+	     ap = append(say_num_as_words(afterpoint.at(i,1)),ap);
+	return append(say_num_as_words(tok.before(".")),
 			cons(strintern("point"),reverse(ap)));
     }
     else if ((tok.matches(make_regex("[A-Z][A-Z]+"))) &&
-	     (tok.length() < 5))       // an acronym
+	     ((!tok.contains(make_regex("[AEIOUY]"))) ||
+	      ((!tok.contains(make_regex("[^AEIOU][AEIOU][^AEIOU]"))) &&
+	       (tok.length() < 5))))       // an acronym
 	return say_as_letters(tok);
     else if (tok.matches(RXdottedabbrev))
 	return say_as_letters(remove_punct(tok));
@@ -307,53 +282,53 @@ static LISP builtin_word_it(LISP utt, EST_Stream_Item &token, EST_String tok)
     else if (only_punc(tok))
 	return stringexplode(tok);
     else if (tok.contains("-"))
-	return l_append(word_it(utt,token,tok.before("-")),
-		      word_it(utt,token,tok.after("-")));
+	return append(word_it(token,tok.before("-")),
+		      word_it(token,tok.after("-")));
     else if (tok.contains(".")) // internet address
     {
 	LISP a=NIL;
 	EST_String tok2 = tok;
 	for ( ; tok2.contains("."); tok2 = tok2.after("."))
-	    a = l_append(a,
-			 l_append(word_it(utt,token,tok2.before(".")),
+	    a = append(a,
+			 append(word_it(token,tok2.before(".")),
 				  cons(strintern("dot"),NIL)));
-	a = l_append(a,word_it(utt,token,tok2));
+	a = append(a,word_it(token,tok2));
 	return a;
     }
     else if (tok.contains("/")) 
-	return l_append(word_it(utt,token,tok.before("/")),
+	return append(word_it(token,tok.before("/")),
 		      cons(strintern("slash"),
-			   word_it(utt,token,tok.after("/"))));
+			   word_it(token,tok.after("/"))));
     else if (tok.contains("&")) 
-	return l_append(word_it(utt,token,tok.before("&")),
+	return append(word_it(token,tok.before("&")),
 		      cons(strintern("ampersand"),
-			   word_it(utt,token,tok.after("&"))));
+			   word_it(token,tok.after("&"))));
     else if (tok.contains("_")) 
-	return l_append(word_it(utt,token,tok.before("_")),
+	return append(word_it(token,tok.before("_")),
 		      cons(strintern("underscore"),
-			   word_it(utt,token,tok.after("_"))));
+			   word_it(token,tok.after("_"))));
     else if (tok.contains("'"))
-	return word_it(utt,token,tok.before("'")+tok.after("'"));
+	return word_it(token,tok.before("'")+tok.after("'"));
     else if (tok.contains("`"))
-	return l_append(word_it(utt,token,tok.before("`")),
-			word_it(utt,token,tok.after("`")));
+	return append(word_it(token,tok.before("`")),
+			word_it(token,tok.after("`")));
     else if (tok.contains("\""))
-	return l_append(word_it(utt,token,tok.before("\"")),
-			word_it(utt,token,tok.after("\"")));
+	return append(word_it(token,tok.before("\"")),
+			word_it(token,tok.after("\"")));
     else if (tok.contains(","))
-	return l_append(word_it(utt,token,tok.before(",")),
-			word_it(utt,token,tok.after(",")));
+	return append(word_it(token,tok.before(",")),
+			word_it(token,tok.after(",")));
     else if (tok.contains("("))
-	return l_append(word_it(utt,token,tok.before("(")),
-			word_it(utt,token,tok.after("(")));
+	return append(word_it(token,tok.before("(")),
+			word_it(token,tok.after("(")));
     else if (tok.contains(")"))
-	return l_append(word_it(utt,token,tok.before(")")),
-			word_it(utt,token,tok.after(")")));
+	return append(word_it(token,tok.before(")")),
+			word_it(token,tok.after(")")));
     else if (tok.matches(make_regex("^[^a-zA-Z].+"))) // incrementally remove 
-	return l_append(say_as_letters(tok.at(0,1)),// num/symbol from front
-			word_it(utt,token,tok.at(1,tok.length()-1)));
+	return append(say_as_letters(tok.at(0,1)),// num/symbol from front
+			word_it(token,tok.at(1,tok.length()-1)));
     else if (tok.matches(make_regex(".+[^a-zA-Z]$"))) // incrementally remove rear
-	return l_append(word_it(utt,token,tok.at(0,tok.length()-1)),
+	return append(word_it(token,tok.at(0,tok.length()-1)),
 			say_as_letters(tok.at((int)tok.length()-1,1)));
     else  // could try harder
 	return say_as_letters(remove_punct(tok));
@@ -412,7 +387,7 @@ static LISP say_as_letters(const EST_String &word)
 	if (name.matches(make_regex("[0-9]")))
 	    CAR(l) = car(say_as_digits(get_c_string(car(l))));
 //	else if (name.matches(make_regex("[^a-zA-Z]")))
-//	    // Not sure, probabably a bug to get here
+//	    // Not sure, probably a bug to get here
 //	    CAR(l) = cons(make_param_str("name","symbol"),
 //			  cons(make_param_lisp("pos",let_pos),NIL));
 	else
@@ -432,7 +407,7 @@ static int only_punc(const EST_String &tok)
     const char *tokch = tok;
 
     for (i=0; i<tok.length(); i++)
-	if (strchr(PunctuationChars,tokch[i]) == NULL)
+      if (strchr((const char *)PunctuationChars,tokch[i]) == NULL)
 	    return FALSE;
 
     return TRUE;
@@ -457,7 +432,7 @@ static LISP say_num_as_ordinal(const EST_String &num)
     
     // Now change the last word to the appropriate ordinal
     for (last=numwords; cdr(last) != NIL; last=cdr(last));
-    char *lastword = get_c_string(car(last));
+    const char *lastword = get_c_string(car(last));
     
     if (streq(lastword,"zero"))  
 	CAR(last) = strintern("zeroth");
@@ -508,14 +483,29 @@ static LISP say_num_as_ordinal(const EST_String &num)
 
 static LISP say_num_as_words(const EST_String &num)
 {
-    return num_2_words(atoi(num));
+    if (num.length() > 9)
+    {
+	if (num(0) == '-')
+	    return cons(strintern("minus"),say_as_digits(num.after("-")));
+	else
+	    return say_as_digits(num);
+    }
+    else	    
+	return num_2_words(atoi(num));
 }
 
 static LISP say_num_as_year(const EST_String &num)
 {
     int iword = atoi(num);
 
-    if (num.matches(make_regex("0[0-9]")))
+    if (num.length() > 4) 
+	return say_num_as_words(num);
+    else if (num.matches(make_regex("00")))
+    {
+	return cons(strintern("o"),
+		    cons(strintern("o"),NIL));
+    }
+    else if (num.matches(make_regex("0[0-9]")))
     {
 	return cons(strintern("o"),
 		    num_2_words(iword));
@@ -525,23 +515,23 @@ static LISP say_num_as_year(const EST_String &num)
     else if ((iword % 1000) < 10)
     {
 	if ((iword % 1000) == 0)
-	    return l_append(num_2_words(iword/1000),
+	    return append(num_2_words(iword/1000),
 			    cons(strintern("thousand"),NIL));
 	else
-	    return l_append(num_2_words(iword/1000),
+	    return append(num_2_words(iword/1000),
 			    cons(strintern("thousand"),
 				 cons(strintern("and"),
 				      num_2_words(iword%1000))));
     }
     else if ((iword % 100) == 0)
-	return l_append(num_2_words(iword/100),
+	return append(num_2_words(iword/100),
 			cons(strintern("hundred"),NIL));
     else if ((iword % 100) < 10)
-	return l_append(num_2_words(iword/100),
+	return append(num_2_words(iword/100),
 			cons(strintern("o"),
 			     num_2_words(iword%100)));
     else 
-	return l_append(num_2_words(iword/100),
+	return append(num_2_words(iword/100),
 			num_2_words(iword%100));
 }
     
@@ -549,7 +539,7 @@ static LISP num_2_words(int iword)
 {
     // Convert number of list of words
     int tens, units;
-    LISP s_tens;
+    LISP s_tens, lang_stype=NIL;
     
     if (iword < 0)
 	return cons(strintern("minus"),num_2_words(-iword));
@@ -602,15 +592,15 @@ static LISP num_2_words(int iword)
     }
     else if (iword < 1000)
     {
-	LISP lang_stype = ft_get_param("Language");
+	lang_stype = ft_get_param("Language");
 	if (streq("americanenglish",get_c_string(lang_stype)))
-	    return l_append(num_2_words(iword/100),
+	    return append(num_2_words(iword/100),
 			    cons(strintern("hundred"),
 				 (((iword % 100) != 0) ?
 				  num_2_words(iword % 100) :
 				  NIL)));
 	else
-	    return l_append(num_2_words(iword/100),
+	    return append(num_2_words(iword/100),
 			    cons(strintern("hundred"),
 				 (((iword % 100) != 0) ?
 				  cons(strintern("and"),
@@ -621,11 +611,11 @@ static LISP num_2_words(int iword)
     // We don't depend on this hack now.
     else if ((iword > 1910) &&
 	     (iword < 2000))   // hacky date condition
-	return l_append(num_2_words(iword/100),
+	return append(num_2_words(iword/100),
 			num_2_words(iword%100));
 #endif
     else if (iword < 1000000)
-	return l_append(num_2_words(iword/1000),
+	return append(num_2_words(iword/1000),
 			cons(strintern("thousand"),
 			     (((iword % 1000) != 0) ?
 			      ((((iword % 1000)/100) == 0) ?
@@ -633,7 +623,7 @@ static LISP num_2_words(int iword)
 			       num_2_words(iword % 1000)) :
 			      NIL)));
     else if (iword >= 1000000)
-	return l_append(num_2_words(iword/1000000),
+	return append(num_2_words(iword/1000000),
 			cons(strintern("million"),
 			     (((iword % 1000000) != 0) ?
 			      num_2_words(iword % 1000000) :
@@ -642,13 +632,13 @@ static LISP num_2_words(int iword)
 	return cons(strintern("bignum"),NIL);
 }
 
-static LISP l_word_it(LISP utt, LISP token, LISP tok)
+static LISP l_word_it(LISP token, LISP tok)
 {
     // Lisp wrap around for word_it
-    EST_Stream_Item *t = GETSTREAMITEMVAL(token);
+    EST_Item *t = get_c_item(token);
     EST_String tok_name = get_c_string(tok);
 
-    return builtin_word_it(utt,*t,tok_name);
+    return builtin_word_it(t,tok_name);
 }
 
 void festival_token_init(void)
@@ -676,8 +666,8 @@ void festival_token_init(void)
   Assign feature token_pos to tokens thats match CART trees in the\n\
   variable token_pos_cart_trees.  These are used for gross level pos\n\
   such as identifying how numbers should be pronunced.");
-    init_subr_3("builtin_english_token_to_words",l_word_it,
-    "(english_token_to_words UTT TOKENSTREAM TOKENNAME)\n\
+    init_subr_2("builtin_english_token_to_words",l_word_it,
+    "(english_token_to_words TOKENSTREAM TOKENNAME)\n\
   Returns a list of words expanded from TOKENNAME.  Note that as this\n\
   function may be called recursively TOKENNAME may not be the name of\n\
   TOKENSTREAM.");
