@@ -66,10 +66,9 @@ USDiphIndex::USDiphIndex() : dihash(1500)
 USDiphIndex::~USDiphIndex() 
 { 
     gc_unprotect(&params);
+    
 }
 
-
-
 void us_check_db()
 {
     if (diph_index == 0)
@@ -77,6 +76,15 @@ void us_check_db()
     diph_index->ts.restart();
 }
 
+void awb_free_diph_index()
+{
+    /* chaising memory leaks */
+    if (diph_index != 0)
+    {
+        delete diph_index;
+        diph_index = 0;
+    }
+}
 
 /*static EST_String get_diphone_name(EST_Item *item,const EST_String dir)
 {
@@ -106,9 +114,9 @@ static EST_String get_diphone_name(EST_Item *item,const EST_String dir)
 
     if (!item)
 	return "";
-    else if ((d1 = item->f(dname+"_"+dir,def)) != "0")
+    else if ((d1 = (EST_String)item->f(dname+"_"+dir,def)) != "0")
 	return d1;
-    else if ((d1 = item->f(dname,def)) != "0")
+    else if ((d1 = (EST_String)item->f(dname,def)) != "0")
 	return d1;
     else
 	return item->f("name","0").string();
@@ -143,7 +151,7 @@ void us_get_diphones(EST_Utterance &utt)
 	utt.relation("Unit")->f.set("sig_ext", diph_index->sig_ext);
     }
 
-    for (p = next(p); p; p = next(p))
+    for (p = inext(p); p; p = inext(p))
     {
 	d = utt.relation("Unit")->append();
 	name2 = get_diphone_name(p,"right");
@@ -153,7 +161,7 @@ void us_get_diphones(EST_Utterance &utt)
     }
 
 //    utt.create_relation("SourceSegments");
-//    for (p = utt.relation("Segment", 1)->head(); p; p = next(p))
+//    for (p = utt.relation("Segment", 1)->head(); p; p = inext(p))
 //    {
 //	d = utt.relation("SourceSegments")->append();
 //	d->set_name(p->name());
@@ -164,6 +172,15 @@ void us_get_diphones(EST_Utterance &utt)
 			    *(utt.relation("Segment", 1)));
 }
 
+LISP us_check_diphone_presence(LISP name)
+{
+    /* requested by "Nicholas Volk" <nvolk@bitlips.fi> */
+    int x = find_diphone_index_simple(get_c_string(name),*diph_index);
+    if ( x < 0 ) 
+	return NIL;
+    else
+	return name;
+}
 
 LISP us_make_group_file(LISP lname, LISP params)
 {
@@ -192,7 +209,7 @@ LISP us_make_group_file(LISP lname, LISP params)
     if ((fp = fopen(group_file, "wb")) == NULL)
 	EST_error("US DB: failed to open group file as temporary file\n");
 
-    for (d = diphone.head(); d; d = next(d))
+    for (d = diphone.head(); d; d = inext(d))
     {
 	sig = wave(d->f("sig"));
 	tr = track(d->f("coefs"));
@@ -208,10 +225,9 @@ LISP us_make_group_file(LISP lname, LISP params)
     fclose(fp);
 
     if ((fp = fopen(index_file, "wb")) == NULL)
-	EST_error("US DB: failed to final group file \"%s\"\n",
+	EST_error("US DB: failed to open group file \"%s\" for writing\n",
 		  (const char *) index_file);
 
-
     fprintf(fp, "EST_File index\n");
     fprintf(fp, "DataType ascii\n");
     fprintf(fp, "NumEntries %d\n", diphone.length());
@@ -222,7 +238,7 @@ LISP us_make_group_file(LISP lname, LISP params)
     fprintf(fp, "sig_file_format %s\n",(const char *)sig_file_format);
     fprintf(fp, "EST_Header_End\n");
 
-    for (d = diphone.head(); d; d = next(d))
+    for (d = diphone.head(); d; d = inext(d))
 	fprintf(fp, "%s %d %d %d\n", (const char *)d->S("name"), 
 		d->I("track_start"), d->I("wave_start"),
 		d->I("middle_frame"));
@@ -233,7 +249,11 @@ LISP us_make_group_file(LISP lname, LISP params)
     int r;
 
     if ((fp_group = fopen(group_file, "rb")) == NULL)
+    {
+	fprintf(stderr,"Unexpected lost temporary group file from \"%s\"\n",
+		(const char *)group_file);
 	return NIL;
+    }
 
     while ((r = fread(buf, sizeof(char), block_size, fp_group)) != 0)
 	fwrite(buf, sizeof(char), r, fp);
@@ -325,7 +345,6 @@ int read_diphone_index(const EST_String &filename,
 		di.diphone[i].set("start", atof(ts.get().string()));
 		di.diphone[i].set("middle", atof(ts.get().string()));
 		di.diphone[i].set("end", ts.get().string());
-
 		if ((di.diphone[i].F("start")>=di.diphone[i].F("middle"))||
 		    (di.diphone[i].F("middle") >= di.diphone[i].F("end")))
 		{
@@ -339,7 +358,7 @@ int read_diphone_index(const EST_String &filename,
 	}
     
 	// now copy reference entries
-	for (i = 0; i < num_entries; ++i)
+	for (i = 0; i < n_num_entries; ++i)
 	    if (di.diphone[i].S("filename").contains("&", 0))
 	    {
 		pointer = di.diphone[i].S("filename").after("&", 0);
@@ -380,10 +399,10 @@ void get_diphone(EST_Item &d)
 	    load_grouped_diphone(unit);
 	else
 	{
-	    if (US_full_coefs)
-		load_full_diphone(unit);
-	    else
-	load_separate_diphone(unit, false, "all");
+	  if (US_full_coefs)
+	    load_full_diphone(unit);
+	  else
+	    load_separate_diphone(unit, false, "all");
 	}
 	diph_index->diphone[unit].set("count", d.I("count", 0) + 1);
     }
@@ -495,7 +514,7 @@ int find_diphone_index(const EST_Item &d)
 	    EST_error("");
 	}
 	else
-	    cout << "UniSyn: using default diphone " << default_diphone << 
+	    cerr << "UniSyn: using default diphone " << default_diphone << 
 		" for " << diname << endl;
 	return index;
     }
@@ -517,7 +536,7 @@ void us_full_cut(EST_Relation &unit)
     int samp_start, samp_end;
     float start_time;
 
-    for (s = unit.head(); s; s = next(s))
+    for (s = unit.head(); s; s = inext(s))
     {
 	sub_coefs = new EST_Track;
 
