@@ -37,7 +37,7 @@
 /* Basic builtin features                                                */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdio.h>
+#include <cstdio>
 #include "festival.h"
 #include "modules.h"
 
@@ -67,10 +67,10 @@ static EST_Val ff_segment_duration(EST_Item *s)
 	    << endl;
 	festival_error();
     }
-    if (prev(n) == 0)
+    if (iprev(n) == 0)
 	return EST_Val(s->F("end", 0));
     else
-	return EST_Val(s->F("end", 0)-(prev(n)->F("end",0)));
+	return EST_Val(s->F("end", 0)-(iprev(n)->F("end",0)));
 }
 
 static EST_Val ff_syllable_duration(EST_Item *s)
@@ -85,11 +85,11 @@ static EST_Val ff_syllable_duration(EST_Item *s)
     else
     {
 	EST_Item *fd = daughter1(n);
-	EST_Item *ld = fd->last();
+	EST_Item *ld = last(fd);
 
 	if (ld == 0)
 	    return val_int0;
-	EST_Item *ps = prev(as(fd,"Segment"));
+	EST_Item *ps = iprev(as(fd,"Segment"));
 	if (ps == 0)
 	    return ld->F("end",0);
 	else
@@ -119,7 +119,7 @@ static EST_Val ff_word_duration(EST_Item *s)
 
 	if (ld == 0)
 	    return val_int0;
-	EST_Item *ps = prev(as(fd,"Segment"));
+	EST_Item *ps = iprev(as(fd,"Segment"));
 	if (ps == 0)
 	    return ld->F("end",0);
 	else
@@ -135,10 +135,10 @@ static EST_Val ff_word_duration(EST_Item *s)
 static EST_Val ff_seg_start(EST_Item *s)
 {
     EST_Item *n = as(s,"Segment");
-    if (prev(n) == 0)
+    if (iprev(n) == 0)
 	return default_val_float;
     else
-	return prev(n)->F("end",0);
+	return iprev(n)->F("end",0);
 }
 
 static EST_Val ff_syl_start(EST_Item *s)
@@ -194,14 +194,14 @@ static EST_Val ff_position_type(EST_Item *s)
 
     if (nn == 0)  // its not really a syllable
 	return EST_Val("single");
-    else if (next(nn) == 0)
+    else if (inext(nn) == 0)
     {
-	if (prev(nn) == 0)
+	if (iprev(nn) == 0)
 	    return EST_Val("single");
 	else
 	    return EST_Val("final");
     }
-    else if (prev(nn) == 0)
+    else if (iprev(nn) == 0)
 	return EST_Val("initial");
     else
 	return EST_Val("mid");
@@ -215,7 +215,7 @@ static EST_Val ff_word_break(EST_Item *w)
     static EST_Val val3 = EST_Val(3);
     static EST_Val val2 = EST_Val(2);
 
-    if ((nn == 0) || (next(nn) != 0))
+    if ((nn == 0) || (inext(nn) != 0))
 	return val_int1;
     else 
     {
@@ -244,7 +244,7 @@ static EST_Val ff_syl_break(EST_Item *s)
 
     if (nn == 0)
 	return val_int1;  // no sylstructure so maybe its standalone
-    else if (next(nn) != 0)  // word internal
+    else if (inext(nn) != 0)  // word internal
 	return val_int0;
     else if (parent(nn) == 0)  // not in a word -- strange
 	return val_int1;
@@ -261,7 +261,7 @@ static EST_Val ff_old_syl_break(EST_Item *s)
 
     if (nn == 0)
 	return val_int1;  // no sylstructure so maybe its standalone
-    else if (next(nn) != 0)  // word internal
+    else if (inext(nn) != 0)  // word internal
 	return val_int0;
     else if (parent(nn) == 0)  // not in a word -- strange
 	return val_int1;
@@ -292,7 +292,7 @@ static EST_Val ff_tobi_accent(EST_Item *s)
     EST_Item *nn = as(s,"Intonation");
     EST_Item *p;
 
-    for (p=daughter1(nn); p; p=next(p))
+    for (p=daughter1(nn); p; p=inext(p))
 	if (p->name().contains("*"))
 	    return EST_Val(p->name());
     return EST_Val("NONE");
@@ -304,7 +304,7 @@ static EST_Val ff_tobi_endtone(EST_Item *s)
     EST_Item *nn = as(s,"Intonation");
     EST_Item *p;
 
-    for (p=daughter1(nn); p; p=next(p))
+    for (p=daughter1(nn); p; p=inext(p))
     {
 	EST_String l = p->name();
 	if ((l.contains("%")) || (l.contains("-")))
@@ -350,7 +350,7 @@ static EST_Val ff_syl_onsetsize(EST_Item *s)
     EST_Item *p;
     int size;
 
-    for (p=daughter1(nn),size=0; p; p=next(p),size++)
+    for (p=daughter1(nn),size=0; p; p=inext(p),size++)
 	if (ph_is_vowel(p->name()))
 	    return EST_Val(size);
     
@@ -365,7 +365,7 @@ static EST_Val ff_syl_vowel(EST_Item *s)
     EST_Item *p;
     int size;
 
-    for (p=daughter1(nn),size=0; p; p=next(p),size++)
+    for (p=daughter1(nn),size=0; p; p=inext(p),size++)
 	if (ph_is_vowel(p->name()))
 	    return EST_Val(p->name());
 
@@ -380,7 +380,7 @@ static EST_Val ff_syl_codasize(EST_Item *s)
     EST_Item *p;
     int size;
 
-    for (p=daughter1(nn)->last(),size=1; p; p=prev(p),size++)
+    for (p=daughtern(nn),size=1; p; p=iprev(p),size++)
 	if (ph_is_vowel(p->name()))
 	    return EST_Val(size);
     
@@ -396,11 +396,11 @@ static EST_Val ff_syl_pc_unvox(EST_Item *s)
 
     if (daughter1(nn) == 0)
 	return val_int0;  // no segments in syllable
-    else if ((ps = prev(daughter1(nn),"Segment")) != 0)
+    else if ((ps = iprev(as(daughter1(nn),"Segment"))) != 0)
 	start = ps->F("end",0);
     unvox = start;
 
-    for (p=daughter1(nn); p != 0; p=next(p))
+    for (p=daughter1(nn); p != 0; p=inext(p))
     {
 	if ((ph_is_vowel(p->name())) ||
 	    (ph_is_voiced(p->name())))
@@ -418,7 +418,7 @@ static EST_Val ff_syl_vowel_start(EST_Item *s)
     EST_Item *nn = as(s,"SylStructure");
     EST_Item *p;
 
-    for (p=daughter1(nn); p != 0; p=next(p))
+    for (p=daughter1(nn); p != 0; p=inext(p))
     {
 	if (ph_is_vowel(p->name()))
 	    return EST_Val(ff_seg_start(p));
@@ -433,7 +433,7 @@ static EST_Val ff_seg_onsetcoda(EST_Item *s)
     EST_Item *nn = as(s,"SylStructure");
     EST_Item *p;
 
-    for (p=next(nn); p; p=next(p))
+    for (p=inext(nn); p; p=inext(p))
 	if (ph_is_vowel(p->name()))
 	    return EST_Val("onset");
     return EST_Val("coda");
@@ -444,7 +444,7 @@ static EST_Val ff_seg_onset_stop(EST_Item *s)
     // 1 if onset of the syllable attached to this segment has a stop
     EST_Item *nn = first(as(s,"SylStructure"));
 
-    for ( ; nn ; nn=next(nn))
+    for ( ; nn ; nn=inext(nn))
     {
 	if (ph_is_vowel(nn->name()))
 	    return val_string0;
@@ -459,7 +459,7 @@ static EST_Val ff_seg_coda_fric(EST_Item *s)
     // 1 if coda of the syllable attached to this segment has a fricative
     EST_Item *nn = last(as(s,"SylStructure"));
 
-    for ( ; nn ; nn=prev(nn))
+    for ( ; nn ; nn=iprev(nn))
     {
 	if (ph_is_vowel(nn->name()))
 	    return val_string0;
@@ -476,7 +476,7 @@ static EST_Val ff_seg_pos_in_syl(EST_Item *s)
     EST_Item *p;
     int pos=0;
 
-    for (p=first(nn); p; p=next(p),pos++)
+    for (p=first(nn); p; p=inext(p),pos++)
 	if (p == nn)
 	    return EST_Val(pos);
     // don't think you can get here
@@ -488,7 +488,7 @@ static EST_Val ff_seg_syl_initial(EST_Item *s)
     // 1 if seg is syllable initial, 0 otherwise.
     EST_Item *nn = as(s,"SylStructure");
 
-    if (prev(nn) == 0)
+    if (iprev(nn) == 0)
 	return val_string1;
     else
 	return val_string0;
@@ -499,7 +499,7 @@ static EST_Val ff_seg_syl_final(EST_Item *s)
     // 1 if seg is syllable initial, 0 otherwise.
     EST_Item *nn = as(s,"SylStructure");
 
-    if (next(nn) == 0)
+    if (inext(nn) == 0)
 	return val_string1;
     else
 	return val_string0;
@@ -512,7 +512,7 @@ static EST_Val ff_syl_pos_in_word(EST_Item *s)
     EST_Item *p;
     int pos=0;
 
-    for (p=first(nn); p; p=next(p),pos++)
+    for (p=first(nn); p; p=inext(p),pos++)
 	if (p == nn)
 	    return EST_Val(pos);
     // don't think you can get here
@@ -526,7 +526,7 @@ static EST_Val ff_pos_in_phrase(EST_Item *s)
     EST_Item *p;
     int pos=0;
 
-    for (p=first(nn); p; p=next(p),pos++)
+    for (p=first(nn); p; p=inext(p),pos++)
 	if (p == nn)
 	    return EST_Val(pos);
     // don't think you can get here
@@ -539,9 +539,9 @@ static EST_Val ff_num_break(EST_Item *s)
     // a new number group
     EST_Item *nn = as(s,"Token");
 
-    if ((next(nn) == 0) &&
+    if ((inext(nn) == 0) &&
 	(parent(nn)->name().matches(RXdouble)) &&
-	(next(parent(nn))->name().matches(RXdouble)))
+	(inext(parent(nn))->name().matches(RXdouble)))
 	return val_string1;
     else
 	return val_string0;
@@ -558,7 +558,7 @@ static EST_Val ff_syl_midpitch(EST_Item *s)
     EST_Item *nn = as(s,"SylStructure");
     EST_Item *p;
 
-    for (p=daughter1(nn); p; p = next(p))
+    for (p=daughter1(nn); p; p = inext(p))
     {
 	if (ph_is_vowel(p->name()))
 	    return ffeature(p,"R:Target.daughter1.f0");
@@ -575,8 +575,10 @@ static EST_Val ff_syl_startpitch(EST_Item *s)
     float pt = ffeature(s,"R:SylStructure.daughter1.R:Segment.p.R:Target.daughter1.f0");
     float tt = ffeature(s,"R:SylStructure.daughter1.R:Segment.R:Target.daughter1.f0");
 	
-    if (pt == 0)
-	return EST_Val(tt);
+    if (pt < 0.1)
+ 	return EST_Val(tt);
+    else if (tt < 0.1)
+	return EST_Val(pt);
     else
 	return EST_Val((tt+pt)/2.0);
 }
@@ -589,8 +591,10 @@ static EST_Val ff_syl_endpitch(EST_Item *s)
     float nt = ffeature(s,"R:SylStructure.daughtern.R:Segment.n.R:Target.daughter1.f0");
     float tt = ffeature(s,"R:SylStructure.daughtern.R:Segment.R:Target.daughter1.f0");
 	
-    if (nt == 0)
-	return EST_Val(tt);
+    if (nt < 0.1)
+ 	return EST_Val(tt);
+    else if (tt < 0.1)
+	return EST_Val(nt);
     else
 	return EST_Val((tt+nt)/2.0);
 }
@@ -603,7 +607,7 @@ static EST_Val ff_seg_pitch(EST_Item *s)
     float smid = ff_seg_mid(s);
     EST_Utterance *u = get_utt(s);
 
-    for (lastt=t=u->relation("Target")->first_leaf(); 
+    for (lastt=t=first_leaf(u->relation("Target")->first()); 
 	 next_leaf(t) != 0; t=next_leaf(t))
     {
 	if (smid <= t->F("pos",0))
@@ -634,12 +638,12 @@ static EST_Val ff_syl_in(EST_Item *s)
     EST_Item *nn = as(s,"Syllable");
     // The first syllable in the phrase
     EST_Item *fsyl = 
-	as(daughter1(first(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	as(daughter1(first(as(parent(s,"SylStructure"),"Phrase")),"SylStructure"),
 	   "Syllable");
     EST_Item *p;
     int count;
 
-    for (count=0,p=nn; p != 0; p=prev(p),count++)
+    for (count=0,p=nn; p != 0; p=iprev(p),count++)
 	if (p == fsyl)
 	    return EST_Val(count);
     return EST_Val(count);
@@ -651,12 +655,12 @@ static EST_Val ff_syl_out(EST_Item *s)
     EST_Item *nn = as(s,"Syllable");
     // The last syllable in the phrase
     EST_Item *lsyl = 
-	as(daughtern(last(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	as(daughtern(last(as(parent(s,"SylStructure"),"Phrase")),"SylStructure"),
 	   "Syllable");
     EST_Item *p;
     int count;
 
-    for (count=0,p=nn; p != 0; p=next(p),count++)
+    for (count=0,p=nn; p != 0; p=inext(p),count++)
 	if (p == lsyl)
 	    return EST_Val(count);
     return EST_Val(count);
@@ -667,31 +671,55 @@ static EST_Val ff_ssyl_in(EST_Item *s)
     // Number of stressed syllables since last phrase break 
     EST_Item *nn = as(s,"Syllable");
     EST_Item *fsyl = 
-	as(daughter1(first(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	as(daughter1(first(as(parent(s,"SylStructure"),"Phrase")),"SylStructure"),
 	   "Syllable");
     EST_Item *p;
     int count;
 
     if (nn == fsyl) return val_int0;
-    for (count=0,p=prev(nn); (p != 0) && (p != fsyl); p = prev(p))
+    for (count=0,p=iprev(nn); (p != 0) && (p != fsyl); p = iprev(p))
 	if (p->F(stressname,0) == 1)
 	    count ++;
     return EST_Val(count);
 }
 
+// This function is modified version of ff_ssyl_in().
+// If first syllable in the current phrase is stressed syllable,
+// ff_ssyl_in() does not count it as stressed syllable.
+// In the following function, this problem is fixed.
+static EST_Val ff_ssyl_in_modified_version(EST_Item *s)
+{
+    // Number of stressed syllables since last phrase break 
+    EST_Item *nn = as(s,"Syllable");
+    EST_Item *fsyl = 
+	as(daughter1(first(as(parent(s,"SylStructure"),"Phrase")),"SylStructure"),
+	   "Syllable");
+    EST_Item *p;
+    int count;
+
+    if (nn == fsyl) return val_int0;
+    for (count=0,p=iprev(nn); (p != 0); p = iprev(p))
+    {
+	if (p->F(stressname,0) == 1)
+	    count ++;
+	if (p == fsyl) break;
+    }
+    return EST_Val(count);
+}
+
 static EST_Val ff_ssyl_out(EST_Item *s)
 {
     // Number of stressed syllables to next phrase break 
     EST_Item *nn = as(s,"Syllable");
     // The last syllable in the phrase
     EST_Item *lsyl = 
-	as(daughtern(last(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	as(daughtern(last(as(parent(s,"SylStructure"),"Phrase")),"SylStructure"),
 	   "Syllable");
     EST_Item *p;
     int count;
 
     if (nn == lsyl) return val_int0;
-    for (count=0,p=next(nn); (p != 0); p=next(p))
+    for (count=0,p=inext(nn); (p != 0); p=inext(p))
     {
 	if (p->F(stressname,0) == 1)
 	    count ++;
@@ -706,31 +734,56 @@ static EST_Val ff_asyl_in(EST_Item *s)
     EST_Item *nn = as(s,"Syllable");
     // The first syllable in the phrase
     EST_Item *fsyl = 
-	as(daughter1(first(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	as(daughter1(first(as(parent(s,"SylStructure"),"Phrase")),"SylStructure"),
 	   "Syllable");
     EST_Item *p;
     int count;
 
     if (nn == fsyl) return val_int0;
-    for (count=0,p=prev(nn); (p != 0) && (p != fsyl); p = prev(p))
+    for (count=0,p=iprev(nn); (p != 0) && (p != fsyl); p = iprev(p))
 	if (ff_syl_accented(p) == 1)
 	    count ++;
     return EST_Val(count);
 }
 
+// This function is modified version of ff_asyl_in().
+// If first syllable in the current phrase is accented syllable,
+// ff_asyl_in() does not count it as accented syllable.
+// In the following function, this problem is fixed.
+static EST_Val ff_asyl_in_modified_version(EST_Item *s)
+{
+    // Number of accented syllables since last phrase break 
+    EST_Item *nn = as(s,"Syllable");
+    // The first syllable in the phrase
+    EST_Item *fsyl = 
+	as(daughter1(first(as(parent(s,"SylStructure"),"Phrase")),"SylStructure"),
+	   "Syllable");
+    EST_Item *p;
+    int count;
+
+    if (nn == fsyl) return val_int0;
+    for (count=0,p=iprev(nn); (p != 0); p = iprev(p))
+    {
+	if (ff_syl_accented(p) == 1)
+	    count ++;
+	if (p == fsyl) break;
+    }
+    return EST_Val(count);
+}
+
 static EST_Val ff_asyl_out(EST_Item *s)
 {
     // Number of accented syllables to next phrase break 
     EST_Item *nn = as(s,"Syllable");
     // The last syllable in the phrase
     EST_Item *lsyl = 
-	as(daughtern(last(parent(s,"SylStructure"),"Phrase"),"SylStructure"),
+	as(daughtern(last(as(parent(s,"SylStructure"),"Phrase")),"SylStructure"),
 	   "Syllable");
     EST_Item *p;
     int count;
 
     if (nn == lsyl) return val_int0;
-    for (count=0,p=next(nn); (p != 0); p=next(p))
+    for (count=0,p=inext(nn); (p != 0); p=inext(p))
     {
 	if (ff_syl_accented(p) == 1)
 	    count ++;
@@ -746,7 +799,7 @@ static EST_Val ff_last_accent(EST_Item *s)
     EST_Item *p;
     int count;
 
-    for (count=0,p=prev(nn); p != 0; p=prev(p),count++)
+    for (count=0,p=iprev(nn); p != 0; p=iprev(p),count++)
 	if (ff_syl_accented(p) == 1)
 	    return EST_Val(count);
 
@@ -760,7 +813,7 @@ static EST_Val ff_next_accent(EST_Item *s)
     EST_Item *p;
     int count;
 
-    for (count=0,p=next(nn); p != 0; p=next(p),count++)
+    for (count=0,p=inext(nn); p != 0; p=inext(p),count++)
 	if (ff_syl_accented(p) == 1)
 	    return EST_Val(count);
 
@@ -774,7 +827,7 @@ static EST_Val ff_sub_phrases(EST_Item *s)
     EST_Item *p;
     int count;
 
-    for (count=0,p=prev(nn); p != 0; p=prev(p))
+    for (count=0,p=iprev(nn); p != 0; p=iprev(p))
     {
 	if (p->name() == "BB")
 	    return EST_Val(count);
@@ -800,7 +853,7 @@ void festival_ff_init(void)
     "Word.word_duration\n\
   The duration of the given stream item.  This is defined as the end of\n\
   last segment in the last syllable (via the SylStructure relation) minus\n\
-  the segment immediate preceeding the first segment in the first syllable.");
+  the segment immediate preceding the first segment in the first syllable.");
     festival_def_nff("segment_start","Segment",ff_seg_start,
     "Segement.segment_start\n\
   The start time of the given segment.");
@@ -817,7 +870,7 @@ void festival_ff_init(void)
     "Segment.segment_end\n\
   The end time of the given segment.");
     festival_def_nff("syllable_end","Syllable",ff_syl_end,
-    "Syllable.word_end\n\
+    "Syllable.syllable_end\n\
   The end time of the given syllable.");
     festival_def_nff("word_end","Word",ff_word_end,
     "Word.word_end\n\
@@ -911,6 +964,10 @@ void festival_ff_init(void)
     festival_def_nff("ssyl_in","Syllable",ff_ssyl_in,
     "Syllable.ssyl_in\n\
   Returns number of stressed syllables since last phrase break, not\n\
+  including this one.");
+    festival_def_nff("ssyl_in_modified_version","Syllable",ff_ssyl_in_modified_version,
+    "Syllable.ssyl_in_modified_version\n\
+  Returns number of stressed syllables since last phrase break, not\n\
   including this one.");
     festival_def_nff("ssyl_out","Syllable",ff_ssyl_out,
     "Syllable.ssyl_out\n\
@@ -920,6 +977,11 @@ void festival_ff_init(void)
     "Syllable.asyl_in\n\
   Returns number of accented syllables since last phrase break, not\n\
   including this one.  Accentedness is as defined by the syl_accented\n\
+  feature.");
+    festival_def_nff("asyl_in_modified_version","Syllable",ff_asyl_in_modified_version,
+    "Syllable.asyl_in_modified_version\n\
+  Returns number of accented syllables since last phrase break, not\n\
+  including this one.  Accentedness is as defined by the syl_accented\n\
   feature.");
     festival_def_nff("asyl_out","Syllable",ff_asyl_out,
     "Syllable.asyl_out\n\
