@@ -38,7 +38,7 @@
 /*                                                                       */
 /*=======================================================================*/
 
-#include <stdio.h>
+#include <cstdio>
 #include "festival.h"
 #include "modules.h"
 #include "text.h"
