@@ -73,11 +73,17 @@
 /*  moved to CMU                                                         */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdlib.h>
+#include <cstdlib>
 #include "EST_math.h"
 #include "festival.h"
 #include "clunits.h"
 
+static EST_String static_unit_prev_move = "unit_prev_move";
+static EST_String static_unit_this_move = "unit_this_move";
+static EST_String static_jscore = "local_join_cost";
+static EST_String static_tscore = "local_target_cost";
+static EST_String static_cscore = "cummulative_unit_score";
+
 static void setup_clunits_params();
 static EST_VTCandidate *TS_candlist(EST_Item *s,EST_Features &f);
 static EST_VTPath *TS_npath(EST_VTPath *p,EST_VTCandidate *c,EST_Features &f);
@@ -104,6 +110,7 @@ static int optimal_coupling = 0;
 static int extend_selections = 0;
 static int clunits_debug = 0;
 static int clunits_log_scores = 0;
+static int clunits_smooth_frames = 0;
 float continuity_weight = 1;
 float f0_join_weight = 0.0;
 float different_prev_pen = 1000.0;
@@ -122,7 +129,7 @@ static LISP clunits_select(LISP utt)
     setup_clunits_params();
 
     f = u->relation("Segment")->head();
-    for (s=f; s; s=next(s))
+    for (s=f; s; s=inext(s))
 	s->set_val("clunit_name",ffeature(s,clunit_name_feat));
 
     if (f)
@@ -137,8 +144,11 @@ static LISP clunits_select(LISP utt)
 	    cerr << "CLUNIT: failed to find path\n";
 	    return utt;
 	}
-	v.copy_feature("unit_this_move");
-	v.copy_feature("unit_prev_move");
+	v.copy_feature(static_unit_this_move);
+	v.copy_feature(static_unit_prev_move);
+	v.copy_feature(static_jscore);
+	v.copy_feature(static_tscore);
+	v.copy_feature(static_cscore);
     }
 
     return utt;
@@ -154,7 +164,7 @@ static LISP clunits_get_units(LISP utt)
     cldb = check_cldb();  // make sure there is one loaded
 
     units = u->create_relation("Unit");
-    for (s=u->relation("Segment")->head(); s != 0; s=next(s))
+    for (s=u->relation("Segment")->head(); s != 0; s=inext(s))
     {
 	EST_Item *unit = units->append();
 	CLunit *db_unit = clunit(s->f("unit_id"));
@@ -162,12 +172,12 @@ static LISP clunits_get_units(LISP utt)
 	unit->set_name(db_unit->name);
 	unit->set("fileid",db_unit->fileid);
 	// These should be modified from the optimal coupling
-	if ((prev(s)) && (s->f_present("unit_this_move")))
+	if ((iprev(s)) && (s->f_present("unit_this_move")))
 	    st = s->F("unit_this_move");
 	else
 	    st = db_unit->start;
-	if ((next(s)) && (next(s)->f_present("unit_prev_move")))
-	    e = next(s)->F("unit_prev_move");
+	if (inext(s) && (inext(s)->f_present("unit_prev_move")))
+	    e = inext(s)->F("unit_prev_move");
 	else
 	    e = db_unit->end;
 	if ((e-st) < 0.011)
@@ -191,7 +201,7 @@ static LISP clunits_get_units(LISP utt)
     // Make it look as much like the diphones as possible for
     // the rest of the code
     ss = u->create_relation("SourceSegments");
-    for (s = u->relation("Segment")->head(); s != 0 ; s = next(s))
+    for (s = u->relation("Segment")->head(); s != 0 ; s = inext(s))
     {
 	EST_Item *d = ss->append();
 	d->set_name(ffeature(s,"clunit_name"));
@@ -212,8 +222,8 @@ static void cl_parse_diphone_times(EST_Relation &diphone_stream,
     float t_time = 0.0, end;
     p_time = 0.0;
     
-    for (s = source_lab.head(), u = diphone_stream.head(); u; u = next(u), 
-	 s = next(s))
+    for (s = source_lab.head(), u = diphone_stream.head(); u;
+         u = inext(u), s = inext(s))
     {
 	pm = track(u->f("coefs"));
 	if (pm == 0)
@@ -249,7 +259,7 @@ static LISP clunits_simple_wave(LISP utt)
     EST_Item *s;
     int size,i,k,c;
 
-    for (size=0,s=u->relation("Unit")->head(); s != 0; s = next(s))
+    for (size=0,s=u->relation("Unit")->head(); s != 0; s = inext(s))
 	size += wave(s->f("sig"))->num_samples();
 
     if (u->relation("Unit")->head())
@@ -259,7 +269,7 @@ static LISP clunits_simple_wave(LISP utt)
     }
     i = w->num_samples();
     w->resize(size); // its maximum size
-    for (s=next(u->relation("Unit")->head()); s; s=next(s))
+    for (s=inext(u->relation("Unit")->head()); s; s=inext(s))
     {
 	w1 = wave(s->f("sig"));
 	// Find last zero crossing
@@ -294,12 +304,11 @@ static LISP clunits_windowed_wave(LISP utt)
     EST_Track *t1 = 0;
     EST_Item *witem = 0;
     EST_Item *s;
-    int size,i,k,wi,samp_idx;
+    int size,i,k,wi,samp_idx, l_samp_idx;
     int width, lwidth;
-    float ltime;
     EST_Wave *www=0;
 
-    for (size=0,s=u->relation("Unit")->head(); s != 0; s = next(s))
+    for (size=0,s=u->relation("Unit")->head(); s != 0; s = inext(s))
 	size += wave(s->f("sig"))->num_samples();
 
     if (u->relation("Unit")->head())
@@ -311,24 +320,24 @@ static LISP clunits_windowed_wave(LISP utt)
     w->resize(size); // its maximum size
     wi=0;
     lwidth = width = 0;
-    for (s=u->relation("Unit")->head(); s; s=next(s))
+    for (s=u->relation("Unit")->head(); s; s=inext(s))
     {
 	w1 = wave(s->f("sig"));
 	t1 = track(s->f("coefs"));
-	samp_idx = 0;
-	ltime = 0;
+
+	l_samp_idx = 0;
 	for (i=0; i < t1->num_frames()-1; i++)
 	{
-	    width = (int)((t1->t(i)-ltime)*w->sample_rate());
-	    if ((i==0) && (lwidth != 0))
+	    samp_idx = (int)(t1->t(i)*w->sample_rate());
+	    width = samp_idx - l_samp_idx;
+	    if (clunits_smooth_frames && (i==0) && (lwidth != 0))
 		width = (width+lwidth)/2;  // not sure if this is worth it
-	    ltime = t1->t(i);
 	    wi += width;
-	    samp_idx += width;
 	    for (k=-width; ((k<width)&&((samp_idx+k)<w1->num_samples())) ;k++)
 		w->a(wi+k) += 
 		    (int)(0.5*(1+cos((PI/(double)(width))*(double)k))*
 			w1->a(samp_idx+k));
+	    l_samp_idx = samp_idx;
 	}
 	lwidth = width;
     }
@@ -346,14 +355,13 @@ static LISP clunits_smoothedjoin_wave(LISP utt)
     EST_Utterance *u = get_c_utt(utt);
     EST_Wave *w = new EST_Wave;
     EST_Wave *w1 = 0;
-    EST_Track *t1 = 0;
     EST_Item *witem = 0;
     EST_Item *s;
     int size,i,wi;
     int samp_end, samp_start;
     EST_Wave *www=0;
 
-    for (size=0,s=u->relation("Unit")->head(); s != 0; s = next(s))
+    for (size=0,s=u->relation("Unit")->head(); s != 0; s = inext(s))
     {
 	samp_end = s->I("samp_end");
 	samp_start = s->I("samp_start");
@@ -368,7 +376,7 @@ static LISP clunits_smoothedjoin_wave(LISP utt)
     }
     w->resize(size); // its maximum size
     wi=0;
-    for (s=u->relation("Unit")->head(); s; s=next(s))
+    for (s=u->relation("Unit")->head(); s; s=inext(s))
     {
 	samp_end = s->I("samp_end");
 	samp_start = s->I("samp_start");
@@ -380,7 +388,6 @@ static LISP clunits_smoothedjoin_wave(LISP utt)
 	       (float)samp_end/(float)w->sample_rate(),
 	       w1->num_samples(),
 	       samp_end); */
-	t1 = track(s->f("coefs"));
 	for (i=samp_start; i<samp_end; i++,wi++)
 	    w->a_no_check(wi) = w1->a_no_check(i);
 /*	printf("%d %f\n",wi,(float)wi/(float)w->sample_rate()); */
@@ -406,6 +413,7 @@ static void setup_clunits_params()
     f0_join_weight = get_param_float("f0_join_weight",clunits_params,0.0);
     clunits_debug = get_param_int("clunits_debug",clunits_params,0);
     clunits_log_scores = get_param_int("log_scores",clunits_params,0);
+    clunits_smooth_frames = get_param_int("smooth_frames",clunits_params,0);
     clunit_name_feat = get_param_str("clunit_name_feat",clunits_params,"name");
     selection_trees = 
 	siod_get_lval("clunits_selection_trees",
@@ -418,7 +426,7 @@ static EST_VTCandidate *TS_candlist(EST_Item *s,EST_Features &f)
     // Use the appropriate CART to select a small group of candidates
     EST_VTCandidate *all_cands = 0;
     EST_VTCandidate *c, *gt;
-    LISP tree,group,l,pd;
+    LISP tree,group,l,pd,cc,ls;
     EST_String name;
     EST_String lookingfor;
     CLunit *u;
@@ -428,8 +436,17 @@ static EST_VTCandidate *TS_candlist(EST_Item *s,EST_Features &f)
     bbb=ccc=0;
 
     lookingfor = s->S("clunit_name");
-    tree = car(cdr(siod_assoc_str(lookingfor,selection_trees)));
-    pd = wagon_pd(s,tree);
+    ls = siod(s);
+
+    cc = siod_get_lval("clunits_cand_hooks",NULL);
+    if (cc)
+	pd = apply_hooks(siod_get_lval("clunits_cand_hooks",NULL),
+			 ls);
+    else
+    {
+	tree = car(cdr(siod_assoc_str(lookingfor,selection_trees)));
+	pd = wagon_pd(s,tree);
+    }
     if (pd == NIL)
     {
 	cerr << "CLUNITS: no predicted class for " << 
@@ -468,7 +485,7 @@ static EST_VTCandidate *TS_candlist(EST_Item *s,EST_Features &f)
 	// An experiment, for all candidates of the previous 
 	// item whose following is of this phone type, include
 	// them as a candidate
-	EST_Item *ppp = prev(s);
+	EST_Item *ppp = iprev(s);
 	if (ppp)
 	{
 	    EST_VTCandidate *lc = vtcand(ppp->f("unit_cands"));
@@ -522,9 +539,6 @@ static EST_VTPath *TS_npath(EST_VTPath *p,EST_VTCandidate *c,EST_Features &f)
     EST_VTPath *np = new EST_VTPath;
     CLunit *u0, *u1;
     float u0_move=0.0, u1_move=0.0;
-    static EST_String static_unit_prev_move = "unit_prev_move";
-    static EST_String static_unit_this_move = "unit_this_move";
-    static EST_String static_lscore = "lscore";
     (void)f;
 
     np->c = c;
@@ -559,11 +573,13 @@ static EST_VTPath *TS_npath(EST_VTPath *p,EST_VTCandidate *c,EST_Features &f)
     if (clunits_log_scores && (cost != 0))
 	cost = log(cost);
 
-//    np->f.set(static_lscore,c->score+cost);
+    np->f.set(static_jscore,cost);
+    np->f.set(static_tscore,c->score);
     if (p==0)
 	np->score = (c->score+cost);
     else
 	np->score = (c->score+cost) + p->score;
+    np->f.set(static_cscore,np->score);
 
     if (clunits_debug > 1)
 	printf("joining cost %f\n",np->score);
@@ -729,7 +745,8 @@ void festival_clunits_init(void)
 {
     // Initialization for clunits selection
 
-    proclaim_module("clunits");
+    proclaim_module("clunits",
+            "Copyright (C) University of Edinburgh and CMU 1997-2010\n");
 
     gc_protect(&clunits_params);
     gc_protect(&selection_trees);
