@@ -52,10 +52,10 @@
 /* (*) PSOLA/TD is a Trade Mark of France Telecom                        */
 /*                                                                       */
 /*=======================================================================*/
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <math.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
+#include <cmath>
 #include "festival.h"
 #include "diphone.h"
 
